(* Title:  Test_Code/test-tool.sml
   Author: Walther Neuper 110226
   (c) due to copyright terms
*)
signature TEST_TOOL =
sig
  val show_ptyps : unit -> unit
  val show_mets : unit -> unit

  val show_pblguhs : unit -> unit
  val sort_pblguhs : unit -> unit
  val show_metguhs : unit -> unit
  val sort_metguhs : unit -> unit

  val show_pt : Ctree.ctree -> unit
  val show_pt_tac : Ctree.ctree -> unit
\<^isac_test>\<open>
  val posterms2str: (Pos.pos' * term * 'a) list -> string
  val postermtacs2str: (Pos.pos' * term * Tactic.input option) list -> string
\<close>
end

(**)
structure Test_Tool(**) : TEST_TOOL(**) =
struct
(**)

fun format_pblID strl = enclose " [" "]" (commas_quote strl);
fun format_pblIDl strll = enclose "[\n" "\n]\n" (space_implode ",\n" (map format_pblID strll));

fun show_ptyps () = (writeln o format_pblIDl o (Store.scan [])) (get_pbls ()); (* for tests *)
fun show_mets () = (writeln o format_pblIDl o (Store.scan [])) (get_pbls ()); (* for tests *)

fun show_pblguhs () =
  (tracing o strs2str o (map linefeed))
    (Check_Unique.collect (#guh : Probl_Def.T -> Check_Unique.id) (get_pbls ()))
fun sort_pblguhs () =
  (tracing o strs2str o (map linefeed))
    (((sort string_ord) o (Check_Unique.collect (#guh : Probl_Def.T -> Check_Unique.id)))
      (get_pbls ()))

fun show_metguhs () =
  (tracing o strs2str o (map linefeed))
    (Check_Unique.collect (#guh : Meth_Def.T -> Check_Unique.id) (get_mets ()))
fun sort_metguhs () =
  (tracing o strs2str o (map linefeed))
    (((sort string_ord) o (Check_Unique.collect (#guh : Meth_Def.T -> Check_Unique.id)))
      (get_mets ()))


(** show a whole Calc.T **)

fun posterm2str (pos, t, _) = "(" ^ Pos.pos'2str pos ^ ", " ^ UnparseC.term t ^ ")"
fun posterms2str pfs = (strs2str' o (map (curry op ^ "\n")) o (map posterm2str)) pfs

fun postermtac2str (pos, t, SOME tac) =
      Pos.pos'2str pos ^ ", " ^ UnparseC.term t ^ "\n" ^ indent 10 ^ Tactic.input_to_string tac
  | postermtac2str (pos, t, NONE) =
      Pos.pos'2str pos ^ ", " ^ UnparseC.term t
fun postermtacs2str pfts = (strs2str' o (map (curry op ^ "\n")) o (map postermtac2str)) pfts

(* WN050225 omits the last step, if pt is incomplete *)
fun show_pt pt = tracing (posterms2str (ME_Misc.get_interval ([], Pos.Frm) ([], Pos.Res) 99999 pt))
fun show_pt_tac pt = tracing (postermtacs2str (ME_Misc.get_interval ([], Pos.Frm) ([], Pos.Res) 99999 pt))

end
